package java.time.temporal;

import java.time.chrono.ChronoZonedDateTime;
import java.time.chrono.ChronoLocalDateTime;
import java.time.chrono.ChronoLocalDate;
import java.time.LocalTime;
import java.time.Duration;

public interface TemporalUnit
{
    public abstract Duration getDuration();
    
    public abstract boolean isDurationEstimated();
    
    public abstract boolean isDateBased();
    
    public abstract boolean isTimeBased();
    
    default public boolean isSupportedBy(Temporal temporal) {
        if ((temporal instanceof LocalTime)) {
            return this.isTimeBased();
        }
        if ((temporal instanceof ChronoLocalDate)) {
            return this.isDateBased();
        }
        if (temporal instanceof ChronoLocalDateTime || temporal instanceof ChronoZonedDateTime) {
            return true;
        }
        try {
            temporal.plus(1L, this);
            return true;
        }
        catch (UnsupportedTemporalTypeException ex) {
            return false;
        }
        catch (RuntimeException ex2) {
            try {
                temporal.plus(-1L, this);
                return true;
            }
            catch (RuntimeException ex3) {
                return false;
            }
        }
    }
    
    public abstract <R extends Temporal> R addTo(R p0, long p1);
    
    public abstract long between(Temporal p0, Temporal p1);
    
    public abstract String toString();
}
